'From Squeak3.10beta of 22 July 2007 [latest update: #7196] on 17 December 2009 at 6:19:23 am'!
RectangleMorph subclass: #SystemProgressMorph
	instanceVariableNames: 'activeSlots bars labels font lock '
	classVariableNames: 'BarHeight BarWidth UniqueInstance Inset '
	poolDictionaries: ''
	category: 'Morphic-Widgets'!

!SystemProgressMorph commentStamp: '<historical>' prior: 0!
An single instance of this morph class is used to display progress while the system is busy, eg. while it receives code updates or does a fileIn. To give the user progress information you don't deal directly with SystemProgressMorph. You keep on using the well established way of progress notification, that has been a long time in the system, is widely used and does not depend on the existence of SystemProgressMorph. For more information on this look at the example in this class or look at the comment of the method displayProgressAt:from:to:during: in class String.

SystemProgressMorph is not meant to be used as a component inside other morphs.

You can switch back to the old style of progress display by disabling the morphicProgressStyle setting in the morphic section of the preferences.!


!SystemProgressMorph methodsFor: 'initialization' stamp: 'laza 7/29/2004 10:26'!
close: aBlock
	| slot |
	slot := aBlock value: SmallInteger maxVal. "This should prevent a redraw"
	self freeSlot: slot
	
! !

!SystemProgressMorph methodsFor: 'initialization' stamp: 'laza 10/9/2009 11:06'!
initialize
	super initialize.
	activeSlots := 0.
	bars := Array new: 10.
	labels := Array new: 10.
	font := Preferences standardMenuFont.
	lock := Semaphore forMutualExclusion.
	self setDefaultParameters;
		setProperty: #morphicLayerNumber toValue: self morphicLayerNumber;
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		cellPositioning: #leftCenter;
		cellInset: 5;
		listCentering: #center;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: Inset;
		minWidth: 150! !

!SystemProgressMorph methodsFor: 'initialization' stamp: 'jrp 7/6/2005 21:45'!
updateColor: aMorph color: aColor intensity: anInteger 
	"update the apareance of aMorph"
	| fill |
	Preferences gradientMenu
		ifFalse: [^ self].

	fill := GradientFillStyle ramp: {0.0 -> Color white. 1 ->aColor}.
	fill radial: false;
		origin: aMorph topLeft;
		direction: 0 @ aMorph height.
	aMorph fillStyle: fill! !

!SystemProgressMorph methodsFor: 'private' stamp: 'laza 10/9/2009 15:21'!
freeSlot: number
	number > 0 ifTrue: [
		lock critical: [
			(bars at: number) delete.
			(labels at: number) delete.
			activeSlots := activeSlots - 1.
			activeSlots = 0
				ifTrue: [self delete]
				ifFalse: [self recenter]]]! !

!SystemProgressMorph methodsFor: 'private' stamp: 'laza 9/9/2009 02:20'!
label: shortDescription min: minValue max: maxValue
	| slot range newBarSize barSize lastRefresh |
	((range := maxValue - minValue) < 0 or: [(slot := self nextSlotFor: shortDescription) = 0])
		ifTrue: [^[:barVal| 0 ]].
	range <= 0 ifTrue: [self removeMorph: (bars at: slot)].
	self recenter.
	self openInWorld.
	barSize := -1. "Enforces a inital draw of the morph"
	lastRefresh := 0.
	^[:barVal |
		barVal isString ifTrue: [
			self setLabel: barVal at: slot.
			self currentWorld displayWorld].
		(barVal isNumber and: [range >= 1 and: [barVal between: minValue and: maxValue]]) ifTrue: [
			newBarSize := (barVal - minValue / range * BarWidth) truncated.
			newBarSize = barSize ifFalse: [
				barSize := newBarSize.
				(bars at: slot) barSize: barSize.
				Time primMillisecondClock - lastRefresh > 25 ifTrue: [
					self currentWorld displayWorld.
					lastRefresh := Time primMillisecondClock]]].
		slot]
! !

!SystemProgressMorph methodsFor: 'private' stamp: 'laza 9/8/2009 06:06'!
nextSlotFor: shortDescription
	| bar slots label |
	lock critical: [
		slots := labels size.
		activeSlots = slots ifTrue: [^0].
		activeSlots := activeSlots + 1.
		1 to: slots do: [:index |
			label := (labels at: index).
			label ifNil: [
				bar := bars at: index put: (SystemProgressBarMorph new extent: BarWidth@BarHeight).
				label := labels at: index put: (StringMorph contents: shortDescription font: font).
				self
					addMorphBack: label;
					addMorphBack: bar.
				^index].
			label owner ifNil: [
				bar := bars at: index.
				label := labels at: index.
				self
					addMorphBack: (label contents: shortDescription);
					addMorphBack: (bar barSize: 0).
				^index]]]
		! !

!SystemProgressMorph methodsFor: 'private' stamp: 'laza 10/9/2009 15:20'!
recenter
	| position |
	"Put ourself in the center of the display"
	self align: self fullBounds center with: Display boundingBox center.
	"Check to see if labels are wider than progress bars. In that case do
	a centered instead of the default left aligned layout."
	position :=	self width > (Inset x * 2 + (self borderWidth * 2) + BarWidth)
					ifTrue: [#topCenter]
					ifFalse: [#leftCenter].
	self cellPositioning: position! !

!SystemProgressMorph methodsFor: 'labelling' stamp: 'laza 9/9/2009 01:15'!
labelAt: progressBlock put: aString
	"Change the label for the given progressBlock to aString."
	progressBlock value: aString! !

!SystemProgressMorph methodsFor: 'labelling' stamp: 'laza 9/7/2009 22:01'!
setLabel: shortDescription at: slot
	(labels at: slot) contents: shortDescription.
	self recenter! !


!SystemProgressMorph class methodsFor: 'class initialization' stamp: 'laza 10/9/2009 11:06'!
initialize
	"SystemProgressMorph initialize; reset"
	BarHeight := 8.
	BarWidth := 300.
	Inset := 30@30! !

!SystemProgressMorph class methodsFor: 'examples' stamp: 'laza 9/8/2009 15:12'!
exampleChangeLabel
	"SystemProgressMorph exampleChangeLabel"
	| classes |
	classes := Smalltalk allClasses copyFrom: 1 to: 100.
	'InitialLabel' 
		displayProgressAt: Display center
		from: 0 to: classes size
		during: [:bar | 1 to: classes size do: [:i |
				bar value: i.
				bar value: i printString, '/', classes size printString, ' ', (classes at: i) printString.
				(Delay forMilliseconds: 100) wait]]
! !

!SystemProgressMorph class methodsFor: 'examples' stamp: 'laza 9/8/2009 22:04'!
exampleLabelOnly
	"SystemProgressMorph exampleLabelOnly"
	| words |
	words := #(zero one two three four five six seven eight nine ten) reversed.
	UIManager default informUserDuring: [:bar |
		words do: [:each|
			bar value: 'Countdown: ', each.
			(Delay forSeconds: 1) wait]].! !

!SystemProgressMorph class methodsFor: 'instance creation' stamp: 'laza 9/7/2009 22:48'!
informUserAt: aPoint during: workBlock
	ProgressInitiationException 
		display: ' '
		at: aPoint 
		from: 0 
		to: 0 
		during: workBlock! !

!SystemProgressMorph class methodsFor: 'instance creation' stamp: 'laza 4/18/2004 21:16'!
label: shortDescription min: minValue max: maxValue
	UniqueInstance ifNil: [UniqueInstance := super new].
	^UniqueInstance label: (shortDescription contractTo: 100) min: minValue asFloat max: maxValue asFloat! !

!SystemProgressMorph class methodsFor: 'instance creation' stamp: 'laza 4/9/2004 10:35'!
reset
	"SystemProgressMorph reset"
	UniqueInstance ifNotNil: [UniqueInstance delete].
	UniqueInstance := nil.! !

!SystemProgressMorph class methodsFor: 'instance creation' stamp: 'ar 6/30/2009 23:24'!
uniqueInstance
	^UniqueInstance ifNil:[super new]! !

SystemProgressMorph initialize!
RectangleMorph subclass: #SystemProgressMorph
	instanceVariableNames: 'activeSlots bars labels font lock'
	classVariableNames: 'BarHeight BarWidth Inset UniqueInstance'
	poolDictionaries: ''
	category: 'Morphic-Widgets'!
